#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = "renaud blanch <http://iihm.imag.fr/blanch/>"
__copyright__ = "Copyright © 2010-2011 - Renaud Blanch"
__licence__   = "GPLv3 <http://www.gnu.org/licenses/gpl.html>"


"""
Pilote study.

see <http://iihm.imag.fr/blanch/projects/distractors/>
"""


# imports ####################################################################

from math import log

from experiment import Experiment
from techniques import (Flat, SemanticPointing, BubbleCursor,
                        DynaSpot, RakeCursor)
from task import W_MacKenzie


# trials #####################################################################

ID = 3.
A = 511
W = W_MacKenzie(ID, A)

rho_max = .7
rhos = []
i = 1
while True:
	try:
		for s in [+1., -1.]:
			rho = i*W/(A+s*W/2.)
			if rho > rho_max:
				raise StopIteration
			rho_l = (i-s/2.)*W/A
			rhos.append((rho, rho_l))
	except StopIteration:
		break
	i += 1


def trials(Technique=SemanticPointing):
	for i in range(10):
		for rho, _ in rhos:
			for A in [511, 1023]:
				yield Technique, (ID, rho, A, 2, False), False

shuffle = True
if shuffle:
	from random import shuffle
	def shuffler(trials):
		def shuffled(*args):
			_trials = list(trials(*args))
			shuffle(_trials)
			for trial in _trials:
				yield trial
		return shuffled
	trials = shuffler(trials)

experiment = Experiment(trials(BubbleCursor))
experiment.run()
