#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = "renaud blanch <http://iihm.imag.fr/blanch/>"
__copyright__ = "Copyright © 2010-2011 - Renaud Blanch"
__licence__   = "GPLv3 <http://www.gnu.org/licenses/gpl.html>"


"""
Illustrates distractors layout uniform density property.

see <http://iihm.imag.fr/blanch/projects/distractors/>
"""


# imports ####################################################################

from OpenGL.GLUT import *
from OpenGL.GL import *

import constants
constants.MIN_A = .1
from task import Task, W_MacKenzie as W


# constants ##################################################################

WINDOW_WIDTH, WINDOW_HEIGHT = 1024, 768

ID = 2
A = 200.
RHO_MIN, RHO_STEP, RHO_MAX  = W(ID, A)/A/2. + .001, 1.+1./100, 1.
rho = RHO_MIN

X = 50
MARGIN = 50
ANIMATION_T = 20

drag = False
animation = "first"


# opengl initialisation ######################################################

glutInit(sys.argv)
glutInitDisplayMode(GLUT_RGBA|GLUT_DOUBLE)
glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT)
glutCreateWindow(sys.argv[0])


# task #######################################################################

from graphics import *

def reshape(W, H):
	global WINDOW_WIDTH, WINDOW_HEIGHT
	WINDOW_WIDTH, WINDOW_HEIGHT = W, H
	glViewport(0, 0, WINDOW_WIDTH, WINDOW_HEIGHT)
	glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
	glOrtho(0, WINDOW_WIDTH, WINDOW_HEIGHT, 0, -1, 1)
	glMatrixMode(GL_MODELVIEW)
	glLoadIdentity()
	glTranslate(0, WINDOW_HEIGHT/2, 0)

rhos_lin = []
rhos_log = []
rhos = []

def display():
	global rho, animation
	if animation:
		rho *= RHO_STEP
		if rho > 1.:
			animation = False
			rho = 1.
	
	task_lin = Task(ID, rho, A, dimension=1, linear=True)
	task_log = Task(ID, rho, A, dimension=1, linear=False)

	if animation == "first":
		rho_log = 1./A*sum(min(A, max(0, X+R))-min(A, max(0, X-R)) for R, X in task_log.distractors)
		rho_lin = 1./A*sum(min(A, max(0, X+R))-min(A, max(0, X-R)) for R, X in task_lin.distractors)
		rhos_log.append(rho_log)
		rhos_lin.append(rho_lin)
		rhos.append(rho)
	
	glClear(GL_COLOR_BUFFER_BIT)

	task_width = WINDOW_WIDTH-WINDOW_HEIGHT/2
	glEnable(GL_CLIP_PLANE1)
	glClipPlane(GL_CLIP_PLANE1, [-1, 0, 0, task_width])

	glPushMatrix()
	glTranslate(X, 0, 0)
	glEnable(GL_CLIP_PLANE0)
	glClipPlane(GL_CLIP_PLANE0, [0, -1, 0, 0])
	draw_task(task_lin)

	glClipPlane(GL_CLIP_PLANE0, [0, 1, 0, 0])
	draw_task(task_log)
	glDisable(GL_CLIP_PLANE0)

	draw_target(1, color=HOME_COLOR, *(5, 0))
		
	# masks
	glColor(0, 0, 0)
	glLineWidth(2)
	for Y in [-WINDOW_HEIGHT/4, WINDOW_HEIGHT/4]:
		glBegin(GL_LINE_LOOP)
		glVertex(-2,  Y-2)
		glVertex(-2,  Y+2)
		glVertex(A+2, Y+2)
		glVertex(A+2, Y-2)
		glEnd()

	glPopMatrix()
	glDisable(GL_CLIP_PLANE1)

	# separations
	glBegin(GL_LINES)
	glVertex(0, 0)
	glVertex(WINDOW_WIDTH, 0)
	glVertex(task_width, -WINDOW_HEIGHT/2)
	glVertex(task_width, WINDOW_HEIGHT/2)
	glEnd()
	
	# graphs
	def tick(x, y):
		X, Y = x*size, y*size
		glBegin(GL_LINE_STRIP)
		glVertex(X, -3)
		glVertex(X, Y)
		glVertex(-3, Y)
		glEnd()
		draw_text("%.2f"%x, X-3, -17)
		draw_text("%.2f"%y, -38, Y-3)
		
	size = WINDOW_HEIGHT/2-2*MARGIN
	for Y, rhos_e in [(0, rhos_lin), (WINDOW_HEIGHT/2, rhos_log)]:
		glPushMatrix()
		glTranslate(task_width+MARGIN+5, Y-MARGIN, 0)
		glScale(1, -1, 1)

		glLineWidth(1)
		glBegin(GL_LINES)
		glVertex(0, 0)
		glVertex(size, 0)
		
		glVertex(0, 0)
		glVertex(0, size)
		
		glVertex(0, 0)
		glVertex(size, size)
		
		glEnd()

		draw_text("rho", size, 6)
		draw_text("rho_e", 4, size-3)

		glLineWidth(3)
		glPushMatrix()
		glScalef(size, size, 1)
		glBegin(GL_LINE_STRIP)
		i_rho = 0
		for i, (rho_r, rho_e) in enumerate(zip(rhos, rhos_e)):
			if rho_r <= rho:
				i_rho = i
			glVertex(rho_r, rho_e)
		glEnd()
		glPopMatrix()

		glLineWidth(1)
		tick(0, 0)
		tick(rhos[i_rho], rhos_e[i_rho])
		glPopMatrix()
	
		
	glutSwapBuffers()

	animate()


def animate(now=False):
	if not animation:
		return
	if not now:
		glutTimerFunc(ANIMATION_T, animate, True)
	else:
		glutPostRedisplay()


def keyboard(k, X, Y):
	global animation
	if k == "a":
		if animation == "first":
			return
		animation = not animation
		animate()
	elif k == "f":
		glutFullScreen()
	elif k in ["q", chr(27)]:
		sys.exit(0)
	

def mouse(button, state, X, Y):
	global drag
	if button == GLUT_LEFT_BUTTON:
		drag = (state == GLUT_DOWN)
		motion(X, Y)


def motion(X, Y):
	global drag, animation, rho
	if not drag:
		return
	if animation == "first":
		return
	animation = False
	task_width = WINDOW_WIDTH-WINDOW_HEIGHT/2
	size = WINDOW_HEIGHT/2-2*MARGIN
	rho = 1.*(X-(task_width+MARGIN+5))/size
	rho = max(rho, RHO_MIN)
	rho = min(rho, RHO_MAX)
	glutPostRedisplay()
	
			
glutReshapeFunc(reshape)
glutDisplayFunc(display)
glutKeyboardFunc(keyboard)
glutMouseFunc(mouse)
glutMotionFunc(motion)


# main #######################################################################

glutMainLoop()
