#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = "renaud blanch <http://iihm.imag.fr/blanch/>"
__copyright__ = "Copyright © 2010-2011 - Renaud Blanch"
__licence__   = "GPLv3 <http://www.gnu.org/licenses/gpl.html>"


"""
Illustrates distractors layout continuity property.

see <http://iihm.imag.fr/blanch/projects/distractors/>
"""


# imports ####################################################################

from math import pi, sqrt

from OpenGL.GLUT import *
from OpenGL.GL import *

from task import Task


# constants ##################################################################

#WINDOW_WIDTH, WINDOW_HEIGHT = 800, 600
WINDOW_WIDTH, WINDOW_HEIGHT = 640, 480


# globals ####################################################################

ID_MIN, ID, ID_MAX = 1., 3., 5.
A = 350.
RHO_MIN, RHO, RHO_MAX = .005, .5, pi/6*sqrt(3)

drag = False


# opengl initialisation ######################################################

glutInit(sys.argv)
glutInitDisplayMode(GLUT_RGBA|GLUT_DOUBLE)
glutInitWindowSize(WINDOW_WIDTH, WINDOW_HEIGHT)
glutCreateWindow(sys.argv[0])


# task #######################################################################

from graphics import *

def reshape(W, H):
	global WINDOW_WIDTH, WINDOW_HEIGHT
	WINDOW_WIDTH, WINDOW_HEIGHT = W, H
	glViewport(0, 0, WINDOW_WIDTH, WINDOW_HEIGHT)
	glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
	glOrtho(0, WINDOW_WIDTH, WINDOW_HEIGHT, 0, -1, 1)
	glMatrixMode(GL_MODELVIEW)
	glLoadIdentity()

def display():	
	glClear(GL_COLOR_BUFFER_BIT)

	glPushMatrix()
	glTranslate(140, WINDOW_HEIGHT/2, 0)
	draw_task(Task(ID, RHO, A, dimension=2))
	draw_target(2, color=HOME_COLOR, *(5, 0, 0))
	glPopMatrix()
	glColor(.5, .5, .5, .5)
	glRectf(22, WINDOW_HEIGHT/2+50, 28, WINDOW_HEIGHT/2-50)
	glRectf(72, WINDOW_HEIGHT/2+50, 78, WINDOW_HEIGHT/2-50)
	glColor(0, 0, 0)
	glBegin(GL_LINES)
	Y_RHO = (WINDOW_HEIGHT/2+50)-100.*RHO
	glVertex(20, Y_RHO)
	glVertex(30, Y_RHO)
	Y_ID = (WINDOW_HEIGHT/2+50)-100.*(ID-ID_MIN)/(ID_MAX-ID_MIN)
	glVertex(70, Y_ID)
	glVertex(80, Y_ID)
	glEnd()
	draw_text("rho",  22, WINDOW_HEIGHT/2-60)
	draw_text("%.2f" % RHO,  32, Y_RHO)
	draw_text("ID",  72, WINDOW_HEIGHT/2-60)
	draw_text("%.2f" % ID, 82, Y_ID)
	glutSwapBuffers()

def keyboard(k, X, Y):
	if k == "f":
		glutFullScreen()
	elif k in ["q", chr(27)]:
		sys.exit(0)

def mouse(button, state, X, Y):
	global drag, X0
	if button == GLUT_LEFT_BUTTON:
		if state == GLUT_DOWN:
			X0 = X
			drag = "A"

			if 0 <= (WINDOW_HEIGHT/2+50)-Y <= 100:
				if 20 <= X <= 30:
					drag = "RHO"
				elif 65 <= X <= 75:
					drag = "ID"
		else:
			drag = False
			
def motion(X, Y):
	global drag, X0, A, RHO, ID
	if drag == "A":
		A /= 1.*max(1, X0-X_OFFSET)
		X0 = X
		A *= 1.*max(1, X0-X_OFFSET)
	
	elif drag in ["RHO", "ID"]:
		u = max(0., min(((WINDOW_HEIGHT/2+50)-Y)/100., 1.))
		if drag == "RHO":
			RHO = max(RHO_MIN, min(u, RHO_MAX))
		elif drag == "ID":
			ID = ID_MIN + u*(ID_MAX - ID_MIN)
	
	glutPostRedisplay()

glutReshapeFunc(reshape)
glutDisplayFunc(display)
glutKeyboardFunc(keyboard)
glutMouseFunc(mouse)
glutMotionFunc(motion)


# main #######################################################################

glutMainLoop()
